# Copyright 2019-2020 Siemens AG
# SPDX-License-Identifier: MIT

import re
import logging
import requests
from datetime import date, timedelta

from .obj import Agents, User, TokenScope, SearchTypes
from .folders import Folders
from .uploads import Uploads
from .jobs import Jobs
from .report import Report
from .exceptions import AuthenticationError, FossologyApiError

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


def fossology_token(
    url, username, password, token_name, token_scope=TokenScope.READ, token_expire=None
):
    """Generate an API token using username/password

    API endpoint: POST /tokens

    :Example:

    >>> from fossology import fossology_token
    >>> from fossology.obj import TokenScope
    >>> token = fossology_token("https://fossology.example.com", "Me", "MyPassword", "MyToken")


    :param url: the URL of the Fossology server
    :param username: name of the user the token will be generated for
    :param password: the password of the user
    :param name: the name of the token
    :param scope: the scope of the token (default: READ)
    :param expire: the expire date of the token (default max. 30 days)
    :type url: string
    :type username: string
    :type password: string
    :type name: string
    :type scope: TokenScope (default TokenScope.READ)
    :type expire: string, e.g. 2019-12-25
    :return: the new token
    :rtype: string
    """
    data = {
        "username": username,
        "password": password,
        "token_name": token_name,
        "token_scope": token_scope.value,
    }
    if token_expire:
        data["token_expire"] = token_expire
    else:
        now = date.today()
        data["token_expire"] = str(now + timedelta(days=30))
    try:
        response = requests.post(url + "/api/v1/tokens", data=data)
        if response.status_code == 201:
            token = response.json()["Authorization"]
            return re.sub("Bearer ", "", token)
        else:
            description = "Error while generating new token"
            raise FossologyApiError(description, response)
    except requests.exceptions.ConnectionError as error:
        exit(f"Server {url} does not seem to be running or is unreachable: {error}")


class Fossology(Folders, Uploads, Jobs, Report):

    """Main Fossology API class

    Authentication against a running Fossology instance is performed using an API token.

    :Example:

    >>> from fossology import Fossology
    >>> foss = Fossology(FOSS_URL, FOSS_TOKEN, username)

    .. note::

        The class instantiation exits if the session with the Fossology server
        can't be established

    :param url: URL of the Fossology instance
    :param token: The API token generated using the Fossology UI
    :param name: The name of the token owner
    :type url: str
    :type token: str
    :type name: str
    :raises AuthenticationError: if the user couldn't be found
    """

    def __init__(self, url, token, name):
        self.host = url
        self.token = token
        self.name = name
        self.users = list()
        self.folders = list()

        self.api = f"{self.host}/api/v1"
        self.session = requests.Session()
        self.session.headers.update({"Authorization": f"Bearer {self.token}"})

        self.user = self._auth()
        self.version = self.get_version()
        self.rootFolder = self.detail_folder(self.user.rootFolderId)
        self.folders = self.list_folders()

        logger.info(
            f"Authenticated as {self.user.name} against {self.host} using API version {self.version}"
        )

    def _auth(self):
        """Perform the first API request and populate user variables

        :return: the authenticated user's details
        :rtype: User
        :raises AuthenticationError: if the user couldn't be found
        """
        self.users = self.list_users()
        for user in self.users:
            if user.name == self.name:
                self.user = user
                return self.user
        logger.error(f"User {self.name} was not found")
        raise AuthenticationError(self.host)

    def close(self):
        self.session.close()

    def get_version(self):
        """Get API version from the server

        API endpoint: GET /version

        :return: the API version string
        :rtype: string
        :raises FossologyApiError: if the REST call failed
        """
        response = self.session.get(f"{self.api}/version")
        if response.status_code == 200:
            return response.json()["version"]
        else:
            description = "Error while getting API version"
            raise FossologyApiError(description, response)

    def detail_user(self, user_id):
        """Get details of Fossology user.

        API Endpoint: GET /users/{id}

        :param id: the ID of the user
        :type id: int
        :return: the requested user's details
        :rtype: User
        :raises FossologyApiError: if the REST call failed
        """
        response = self.session.get(f"{self.api}/users/{user_id}")
        if response.status_code == 200:
            user_agents = None
            user_details = response.json()
            if user_details.get("agents"):
                user_agents = Agents.from_json(user_details["agents"])
            user = User.from_json(user_details)
            user.agents = user_agents
            return user
        else:
            description = f"Error while getting details for user {user_id}"
            raise FossologyApiError(description, response)

    def list_users(self):
        """ List all users from the Fossology instance

        API Endpoint: GET /users

        :return: the list of users
        :rtype: list of User
        :raises FossologyApiError: if the REST call failed
        """
        response = self.session.get(f"{self.api}/users")
        if response.status_code == 200:
            users_list = list()
            for user in response.json():
                if user.get("name") == "Default User":
                    continue
                if user.get("email"):
                    foss_user = User.from_json(user)
                    if agents := user.get("agents"):
                        foss_user.agents = Agents.from_json(agents)
                    users_list.append(foss_user)
            return users_list
        else:
            description = f"Unable to get a list of users from {self.host}"
            raise FossologyApiError(description, response)

    def delete_user(self, user):
        """Delete a Fossology user.

        API Endpoint: DELETE /users/{id}

        :param user: the user to be deleted
        :type user: User
        :raises FossologyApiError: if the REST call failed
        """
        response = self.session.delete(f"{self.api}/users/{user.id}")
        print(response.json())
        if response.status_code == 202:
            return
        else:
            description = f"Error while deleting user {user.name} ({user.id})"
            raise FossologyApiError(description, response)

    def search(
        self,
        searchType=SearchTypes.ALLFILES,
        filename=None,
        tag=None,
        filesizemin=None,
        filesizemax=None,
        license=None,
        copyright=None,
    ):
        """Search for a specific file

        API Endpoint: GET /search

        :param searchType: Limit search to: directory, allfiles (default), containers
        :param filename: Filename to find, can contain % as wild-card
        :param tag: tag to find
        :param filesizemin: Min filesize in bytes
        :param filesizemax: Max filesize in bytes
        :param license: License search filter
        :param copyright: Copyright search filter
        :type searchType: SearchType Enum
        :type filename: string
        :type tag: string
        :type filesizemin: int
        :type filesizemax: int
        :type license: string
        :type copyright: string
        :return: list of items corresponding to the search criteria
        :rtype: JSON
        :raises FossologyApiError: if the REST call failed
        """
        headers = {"searchType": searchType.value}
        if filename:
            headers["filename"] = filename
        if tag:
            headers["tag"] = tag
        if filesizemin:
            headers["filesizemin"] = filesizemin
        if filesizemax:
            headers["filesizemax"] = filesizemax
        if license:
            headers["license"] = license
        if copyright:
            headers["copyright"] = copyright

        response = self.session.get(f"{self.api}/search", headers=headers)
        if response.status_code == 200:
            return response.json()
        else:
            description = "Unable to get a result with the given search criteria"
            raise FossologyApiError(description, response)



# Copyright 2019-2020 Siemens AG
# SPDX-License-Identifier: MIT

import re
import time
import logging

from tenacity import retry, TryAgain, stop_after_attempt, retry_if_exception_type
from .exceptions import FossologyApiError

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


class Report:
    """Class dedicated to all "report" related endpoints"""

    @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(3))
    def generate_report(self, upload, report_format=None):
        """Generate a report for a given upload

        API Endpoint: GET /report

        :param upload: the upload which report will be generated
        :param format: the report format (default ReportFormat.READMEOSS)
        :type upload: Upload
        :type format: ReportFormat
        :return: the report id
        :rtype: int
        :raises FossologyApiError: if the REST call failed
        """
        headers = {"uploadId": str(upload.id)}
        if report_format:
            headers["reportFormat"] = report_format.value
        else:
            headers["reportFormat"] = "readmeoss"

        response = self.session.get(f"{self.api}/report", headers=headers)
        if response.status_code == 201:
            report_id = re.search("[0-9]*$", response.json()["message"])
            return report_id[0]
        elif response.status_code == 503:
            wait_time = response.headers["Retry-After"]
            logger.debug(f"Retry generate report after {wait_time} seconds")
            time.sleep(int(wait_time))
            raise TryAgain
        else:
            description = f"Report generation for upload {upload.name} failed"
            raise FossologyApiError(description, response)

    @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(3))
    def download_report(self, report_id, as_zip=False):
        """Download a report

        API Endpoint: GET /report/{id}

        :Example:

        >>> from fossology.api import Fossology
        >>>
        >>> foss = Fossology(FOSS_URL, FOSS_TOKEN, username)
        >>>
        >>> # Generate a report for upload 1
        >>> report_id = foss.generate_report(foss.detail_upload(1))
        >>> report_content = foss.download_report(report_id, as_zip=True)
        >>> with open(filename, "w+") as report_file:
        >>>     report_file.write(report_content)

        :param report_id: the id of the generated report
        :param as_zip: control if the report should be generated as ZIP file (default False)
        :type report_id: int
        :type as_zip: boolean
        :raises FossologyApiError: if the REST call failed
        """
        if as_zip:
            headers = {"Accept": "application/zip"}
        else:
            headers = {"Accept": "text/plain"}

        response = self.session.get(f"{self.api}/report/{report_id}", headers=headers)
        if response.status_code == 200:
            return response.text
        elif response.status_code == 503:
            wait_time = response.headers["Retry-After"]
            logger.debug(f"Retry get report after {wait_time} seconds")
            time.sleep(int(wait_time))
            raise TryAgain
        else:
            description = "Download of report {report_id} failed"
            raise FossologyApiError(description, response)



# Copyright 2020 Siemens AG
# SPDX-License-Identifier: MIT

import json
import time
import logging
from tenacity import retry, retry_if_exception_type, stop_after_attempt, TryAgain

from .obj import Upload, Summary
from .exceptions import FossologyApiError

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


class Uploads:
    """Class dedicated to all "uploads" related endpoints"""

    # Retry until the unpack agent is finished
    @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(10))
    def detail_upload(self, upload_id):
        """Get detailled information about an upload

        API Endpoint: GET /uploads/{id}

        :param upload_id: the id of the upload
        :type: int
        :return: the upload data
        :rtype: Upload
        :raises FossologyApiError: if the REST call failed
        """
        response = self.session.get(f"{self.api}/uploads/{upload_id}")
        if response.status_code == 200:
            logger.debug(f"Got details for upload {upload_id}")
            return Upload.from_json(response.json())
        elif response.status_code == 503:
            logger.debug(
                f"Unpack agent for {upload_id} didn't start yet, is the scheduler running?"
            )
            time.sleep(20)
            raise TryAgain
        else:
            description = f"Error while getting details for upload {upload_id}"
            raise FossologyApiError(description, response)

    def upload_file(  # noqa: C901
        self,
        folder,
        file=None,
        vcs=None,
        url=None,
        description=None,
        access_level=None,
        ignore_scm=False,
        group=None,
    ):
        """Upload a file to FOSSology

        API Endpoint: POST /uploads

        :Example for a file upload:

        >>> from fossology import Fossology
        >>> from fossology.obj import AccessLevel
        >>> foss = Fossology(FOSS_URL, FOSS_TOKEN, username)
        >>> my_upload = foss.upload_file(
                foss.rootFolder,
                file="my-package.zip",
                description="My product package",
                access_level=AccessLevel.PUBLIC,
            )

        :Example for a VCS upload:

        >>> vcs = {
                "vcsType": "git",
                "vcsUrl": "https://github.com/fossology/fossology-python",
                "vcsName": "fossology-python-github-master",
                "vcsUsername": "",
                "vcsPassword": "",
            }
        >>> vcs_upload = foss.upload_file(
                foss.rootFolder,
                vcs=vcs,
                description="Upload from VCS",
                access_level=AccessLevel.PUBLIC,
            )

        :Example for a URL upload:

        >>> url = {
                "url": "https://github.com/fossology/fossology-python/archive/master.zip",
                "name": "fossology-python-master.zip",
                "accept": "zip",
                "reject": "",
                "maxRecursionDepth": "1",
            }
        >>> url_upload = foss.upload_file(
                foss.rootFolder,
                url=url,
                description="Upload from URL",
                access_level=AccessLevel.PUBLIC,
            )

        :param folder: the upload Fossology folder
        :param file: the local path of the file to be uploaded
        :param vcs: the VCS specification to upload from an online repository
        :param url: the URL specification to upload from a url
        :param description: description of the upload (default: None)
        :param access_level: access permissions of the upload (default: protected)
        :param ignore_scm: ignore SCM files (Git, SVN, TFS) (default: False)
        :param group: the group name to chose while uploading the file (default: None)
        :type folder: Folder
        :type file: string
        :type vcs: dict()
        :type url: dict()
        :type description: string
        :type access_level: AccessLevel
        :type ignore_scm: boolean
        :type group: string
        :return: the upload data
        :rtype: Upload
        :raises FossologyApiError: if the REST call failed
        """
        headers = {"folderId": str(folder.id)}
        if description:
            headers["uploadDescription"] = description
        if access_level:
            headers["public"] = access_level.value
        if ignore_scm:
            headers["ignoreScm"] = "true"
        if group:
            headers["groupName"] = group

        if file:
            headers["uploadType"] = "server"
            with open(file, "rb") as fp:
                files = {"fileInput": fp}
                response = self.session.post(
                    f"{self.api}/uploads", files=files, headers=headers
                )
        elif vcs:
            headers["uploadType"] = "vcs"
            data = json.dumps(vcs)
            headers["Content-Type"] = "application/json"
            response = self.session.post(
                f"{self.api}/uploads", data=data, headers=headers
            )
        elif url:
            headers["uploadType"] = "url"
            data = json.dumps(url)
            headers["Content-Type"] = "application/json"
            response = self.session.post(
                f"{self.api}/uploads", data=data, headers=headers
            )
        else:
            logger.debug("Neither VCS or File option given, not uploading anything")
            return

        if response.status_code == 201:
            try:
                upload = self.detail_upload(response.json()["message"])
                logger.info(
                    f"Upload {upload.uploadname} ({upload.filesize}) "
                    f"has been uploaded on {upload.uploaddate}"
                )
                return upload
            except TryAgain:
                if file:
                    source = f"{file}"
                elif vcs:
                    source = vcs.get("vcsName")
                else:
                    source = url.get("name")
                description = f"Upload of {source} failed"
                raise FossologyApiError(description, response)

    @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(3))
    def upload_summary(self, upload):
        """Get clearing information about an upload

        API Endpoint: GET /uploads/{id}/summary

        :param upload: the upload to gather data from
        :type: Upload
        :return: the upload summary data
        :rtype: Summary
        :raises FossologyApiError: if the REST call failed
        """
        response = self.session.get(f"{self.api}/uploads/{upload.id}/summary")
        if response.status_code == 200:
            return Summary.from_json(response.json())
        elif response.status_code == 503:
            logger.debug(
                f"Unpack agent for {upload.uploadname} (id={upload.id}) didn't start yet"
            )
            time.sleep(3)
            raise TryAgain
        else:
            description = f"No summary for upload {upload.uploadname} (id={upload.id})"
            raise FossologyApiError(description, response)

    @retry(retry=retry_if_exception_type(TryAgain), stop=stop_after_attempt(3))
    def upload_licenses(self, upload, agent=None, containers=False):
        """Get clearing information about an upload

        API Endpoint: GET /uploads/{id}/licenses

        :param upload: the upload to gather data from
        :param agent: the license agent to use (default: LicenseAgent.MONK)
        :param containers: wether to show containers or not (default: False)
        :type upload: Upload
        :type agent: LicenseAgent
        :type containers: boolean
        :return: the licenses found by the specified agent
        :rtype: list of licenses as JSON object
        :raises FossologyApiError: if the REST call failed
        """
        if agent:
            agent = agent.value
        else:
            agent = "nomos"
        if containers:
            containers = "true"
            response = self.session.get(
                f"{self.api}/uploads/{upload.id}/licenses?agent={agent}&containers={containers}"
            )
        else:
            response = self.session.get(
                f"{self.api}/uploads/{upload.id}/licenses?agent={agent}"
            )

        if response.status_code == 200:
            return response.json()
        elif response.status_code == 412:
            logger.info(
                f"Agent {agent} has not been scheduled for {upload.uploadname} (id={upload.id}): "
                f"{response.json()['message']}"
            )
            return
        elif response.status_code == 503:
            logger.debug(
                f"Unpack agent for {upload.uploadname} (id={upload.id}) didn't start yet"
            )
            time.sleep(3)
            raise TryAgain
        else:
            description = f"No licenses for upload {upload.uploadname} (id={upload.id})"
            raise FossologyApiError(description, response)

    def delete_upload(self, upload, group=None):
        """Delete an upload

        API Endpoint: DELETE /uploads/{id}

        :param upload: the upload to be deleted
        :param group: the group name to chose while deleting the upload (default None)
        :type upload: Upload
        :type group: string
        :raises FossologyApiError: if the REST call failed
        """
        headers = {}
        if group:
            headers["groupName"] = group
        response = self.session.delete(
            f"{self.api}/uploads/{upload.id}", headers=headers
        )
        if response.status_code == 202:
            logger.info(f"Upload {upload.id} has been scheduled for deletion")
        else:
            description = f"Unable to delete upload {upload.id}"
            raise FossologyApiError(description, response)

    def list_uploads(self):
        """Get all uploads available to the registered user

        API Endpoint: GET /uploads

        :return: a list of uploads
        :rtype: list of Upload
        :raises FossologyApiError: if the REST call failed
        """
        response = self.session.get(f"{self.api}/uploads")
        if response.status_code == 200:
            uploads_list = list()
            for upload in response.json():
                uploads_list.append(Upload.from_json(upload))
            return uploads_list
        else:
            description = "Unable to retrieve the list of uploads"
            raise FossologyApiError(description, response)

    def move_upload(self, upload, folder, group=None):
        """Move an upload to another folder

        API Endpoint: PATCH /uploads/{id}

        :param upload: the Upload to be copied in another folder
        :param folder: the destination Folder
        :param group: the group name to chose while deleting the upload (default None)
        :type upload: Upload
        :type folder: Folder
        :type group: string
        :raises FossologyApiError: if the REST call failed
        """
        headers = {"folderId": str(folder.id)}
        if group:
            headers["groupName"] = group
        response = self.session.patch(
            f"{self.api}/uploads/{upload.id}", headers=headers
        )
        if response.status_code == 202:
            logger.info(f"Upload {upload.uploadname} has been moved to {folder.name}")
        else:
            description = f"Unable to move upload {upload.uploadname} to {folder.name}"
            raise FossologyApiError(description, response)

    def copy_upload(self, upload, folder):
        """Copy an upload in another folder

        API Endpoint: PUT /uploads/{id}

        :param upload: the Upload to be copied in another folder
        :param folder: the destination Folder
        :type upload: Upload
        :type folder: Folder
        :raises FossologyApiError: if the REST call failed
        """
        headers = {"folderId": str(folder.id)}
        response = self.session.put(f"{self.api}/uploads/{upload.id}", headers=headers)
        if response.status_code == 202:
            logger.info(f"Upload {upload.uploadname} has been copied to {folder.name}")
        else:
            description = f"Unable to copy upload {upload.uploadname} to {folder.name}"
            raise FossologyApiError(description, response)



# Copyright 2019-2020 Siemens AG
# SPDX-License-Identifier: MIT

import json
import time
import logging

from .obj import Job
from .exceptions import FossologyApiError

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


class Jobs:
    """Class dedicated to all "jobs" related endpoints"""

    def list_jobs(self, page_size=20, pages=1, upload=None):
        """Get all available jobs

        API Endpoint: GET /jobs

        The answer is limited to the first page of 20 results by default

        :param page_size: the maximum number of results per page
        :param pages: the number of pages to be retrieved
        :param upload: list only jobs of the given upload (default: None)
        :type page_size: int (default: 20)
        :type pages: int (default: 1)
        :type upload: Upload
        :return: the jobs data
        :rtype: list of Job
        :raises FossologyApiError: if the REST call failed
        """
        headers = {"limit": str(page_size), "pages": str(pages)}
        if upload:
            response = self.session.get(
                f"{self.api}/jobs?upload={upload.id}", headers=headers
            )
        else:
            response = self.session.get(f"{self.api}/jobs", headers=headers)
        if response.status_code == 200:
            jobs_list = list()
            for job in response.json():
                jobs_list.append(Job.from_json(job))
            return jobs_list
        else:
            description = "Getting the list of jobs failed"
            raise FossologyApiError(description, response)

    def detail_job(self, job_id, wait=False, timeout=30):
        """Get detailled information about a job

        API Endpoint: GET /jobs/{id}

        :param job_id: the id of the job
        :param wait: wait until the job is finisched (default: False)
        :param timeout: stop waiting after x seconds (default: 30)
        :type: int
        :type wait: boolean
        :type timeout: 30
        :return: the job data
        :rtype: Job
        :raises FossologyApiError: if the REST call failed
        """
        response = self.session.get(f"{self.api}/jobs/{job_id}")
        if wait:
            if response.status_code == 200:
                job = Job.from_json(response.json())
                if job.status == "Completed":
                    logger.debug(f"Job {job_id} has completed")
                    return job
            else:
                description = f"Error while getting details for job {job_id}"
                raise FossologyApiError(description, response)
            logger.debug(f"Waiting for job {job_id} to complete")
            time.sleep(timeout)
            response = self.session.get(f"{self.api}/jobs/{job_id}")

        if response.status_code == 200:
            logger.debug(f"Got details for job {job_id}")
            return Job.from_json(response.json())
        else:
            description = f"Error while getting details for job {job_id}"
            raise FossologyApiError(description, response)

    def schedule_jobs(self, folder, upload, spec, wait=False, timeout=30):
        """Schedule jobs for a specific upload

        API Endpoint: POST /jobs

        Job specifications `spec` are added to the request body,
        following options are available:

        >>> {
        >>>     "analysis": {
        >>>         "bucket": True,
        >>>         "copyright_email_author": True,
        >>>         "ecc": True,
        >>>         "keyword": True,
        >>>         "monk": True,
        >>>         "mime": True,
        >>>         "monk": True,
        >>>         "nomos": True,
        >>>         "ojo": True,
        >>>         "package": True,
        >>>         "specific_agent": True,
        >>>     },
        >>>     "decider": {
        >>>         "nomos_monk": True,
        >>>         "bulk_reused": True,
        >>>         "new_scanner": True,
        >>>         "ojo_decider": True
        >>>     },
        >>>     "reuse": {
        >>>         "reuse_upload": 0,
        >>>         "reuse_group": 0,
        >>>         "reuse_main": True,
        >>>         "reuse_enhanced": True
        >>>     }
        >>> }

        :param folder: the upload folder
        :param upload: the upload for which jobs will be scheduled
        :param spec: the job specification
        :param wait: wait for the scheduled job to finish (default: False)
        :param timeout: stop waiting after x seconds (default: 30)
        :type upload: Upload
        :type folder: Folder
        :type spec: dict
        :type wait: boolean
        :type timeout: 30
        :return: the job id
        :rtype: Job
        :raises FossologyApiError: if the REST call failed
        """
        headers = {
            "folderId": str(folder.id),
            "uploadId": str(upload.id),
            "Content-Type": "application/json",
        }
        response = self.session.post(
            f"{self.api}/jobs", headers=headers, data=json.dumps(spec)
        )
        if response.status_code == 201:
            detailled_job = self.detail_job(
                response.json()["message"], wait=wait, timeout=timeout
            )
            return detailled_job
        else:
            description = "Scheduling jobs for upload {upload.uploadname} failed"
            raise FossologyApiError(description, response)



# Copyright 2019-2020 Siemens AG
# SPDX-License-Identifier: MIT

from json.decoder import JSONDecodeError


class Error(Exception):
    """Base class for exceptions in this module."""

    pass


class AuthenticationError(Error):
    """Authentication error"""

    def __init__(self, url):
        self.url = url
        self.message = (
            f"An error occurred during authentication against {self.url}\n"
            f"Check your API Token and try again"
        )


class AuthorizationError(Error):
    """Authorization error"""

    def __init__(self, description, response):
        try:
            message = response.json().get("message")
        except JSONDecodeError:
            message = response.text
        self.message = f"{description}: {message} ({response.status_code})"


class FossologyApiError(Error):
    """Error during a Fossology GET request"""

    def __init__(self, description, response):
        try:
            message = response.json().get("message")
        except JSONDecodeError:
            message = response.text
        self.message = f"{description}: {message} ({response.status_code})"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fossology']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.22.0', 'tenacity>=6.0.0']

setup_kwargs = {
    'name': 'fossology',
    'version': '0.1.1',
    'description': 'A library to automate Fossology from Python scripts',
    'long_description': '|License| |PyPI Version| |Python Version| |Static Checks| |Fossology Tests| |Coverage|\n\n.. |License| image:: https://img.shields.io/badge/license-MIT-blue.svg\n   :target: https://github.com/deveaud-m/fossology-python/LICENSE.md\n\n.. |PyPI Version| image:: https://badge.fury.io/py/fossology.svg\n   :target: https://pypi.org/project/fossology\n\n.. |Python Version| image:: https://img.shields.io/badge/python-3.6%2C3.7%2C3.8-blue?logo=python\n   :target: https://www.python.org/doc/versions/\n\n.. |Static Checks| image:: https://github.com/deveaud-m/fossology-python/workflows/Static%20Checks/badge.svg\n   :target: https://github.com/deveaud-m/fossology-python/actions?query=workflow%3A%22Static+Checks%22\n   \n.. |Fossology Tests| image:: https://github.com/deveaud-m/fossology-python/workflows/Fossology%20Tests/badge.svg\n   :target: https://github.com/deveaud-m/fossology-python/actions?query=workflow%3A%22Fossology+Tests%22\n\n.. |Coverage| image:: https://codecov.io/gh/fossology/fossology-python/branch/master/graph/badge.svg\n   :target: https://codecov.io/gh/fossology/fossology-python\n\nA simple wrapper for the Fossology REST API.\n\nSee `the OpenAPI specification <https://raw.githubusercontent.com/fossology/fossology/master/src/www/ui/api/documentation/openapi.yaml>`_ used to implement this library.\n\n   Compatible with API version 1.0.16\n\nDocumentation\n=============\n\nSee `fossology-python on Github Pages <https://fossology.github.io/fossology-python>`_.\n\nUsage\n=====\n\nInstallation\n------------\n\n   This project is available as `Python package on PyPi.org <https://pypi.org/project/fossology/>`_.\n\n-  Install fossology and required dependencies:\n\n   .. code:: shell\n\n      pip install fossology requests\n\nUsing the API\n-------------\n\n-  Get a REST API token either from the Fossology server under "User->Edit user account" or generate a token using the method available in this library:\n\n   .. code:: Python\n\n      from fossology import fossology_token\n      from fossology.obj import TokenScope\n\n      FOSSOLOGY_SERVER = "https://fossology.example.com/"\n      FOSSOLOGY_USER = "fossy"\n      FOSSOLOGY_PASSWORD = "fossy"\n      TOKEN_NAME = "fossy_token"\n\n      token = fossology_token(\n            FOSSOLOGY_SERVER,\n            FOSSOLOGY_USER,\n            FOSSOLOGY_PASSWORD,\n            TOKEN_NAME,\n            TokenScope.WRITE\n      )\n\n-  Start using the API:\n\n   .. code:: python\n\n      from fossology import Fossology\n\n      foss = Fossology(\n            FOSSOLOGY_SERVER,\n            token,\n            FOSSOLOGY_USER\n      )\n\n\nContribute\n==========\n\nDevelop\n-------\n\n-  All contributions in form of bug reports, feature requests or merge requests!\n\n-  Use proper\n   `docstrings <https://realpython.com/documenting-python-code/>`__ to\n   document functions and classes\n\n-  Don\'t forget to extend the minimal `testsuite <test.py>`_ with the\n   new functions/classes\n\n-  The **documentation website** can automatically be generated by the `Sphinx autodoc\n   extension <http://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html>`_\n\n\nBuild\n-----\n\n- You can build the PyPi package using `poetry <https://poetry.eustace.io/>`_:\n\n  .. code:: shell\n\n    poetry build\n\n- Build documentation (the generated static site must be pushed to the **gh-pages** branch):\n\n  .. code:: shell\n\n     git clone -b gh-pages git@github.com:fossology/fossology-python.git docs/\n     poetry run sphinx-build docs-source docs/\n     cd docs/\n     # Create a new branch to be merged into gh-pages and commit your changes\n\n- Cleanup builds:\n\n  .. code:: shell\n\n     rm -r dist/ build/ docs/\n\n\nTest\n----\n\nThe testsuite available in this project expects a running Fossology instance under the hostname **fossology** with the default admin user "fossy".\n\n- Use the latest Fossology container from `Docker hub <https://hub.docker.com/r/fossology/fossology>`_:\n\n  .. code:: shell\n\n    docker pull fossology/fossology\n    docker run --name fossology -p 80:80 fossology/fossology\n\n- Start the complete test suite or a specific test case (and generate coverage report):\n\n  .. code:: shell\n\n     poetry run coverage run --source=fossology tests/tests.py\n     poetry run coverage report -m\n     poetry run coverage html\n',
    'author': 'Marion Deveaud',
    'author_email': 'marion.deveaud@siemens.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/fossology/fossology-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
